@@ -22,80 +22,29 @@ fn issue_warning(msg: &str) {
 /// This contains the default configuration lua file
 const DEFAULT_CONFIG: &str = include_str!("../config/.oxrc");
 
+/// Default plug-in code to use
+const PAIRS: &str = include_str!("../plugins/pairs.lua");
+const AUTOINDENT: &str = include_str!("../plugins/autoindent.lua");
+
 /// This contains the code for setting up plug-in infrastructure
-pub const PLUGIN_BOOTSTRAP: &str = r#"
-home = os.getenv("HOME") or os.getenv("USERPROFILE")
-
-function file_exists(file_path)
-    local file = io.open(file_path, "r")
-    if file then
-        file:close()
-        return true
-    else
-        return false
-    end
-end
-
-plugins = {}
-plugin_issues = false
-
-function load_plugin(base)
-    path_cross = base
-    path_unix = home .. "/.config/ox/" .. base
-    path_win = home .. "/ox/" .. base
-    if file_exists(path_cross) then
-        path = path_cross
-    elseif file_exists(path_unix) then
-        path = path_unix
-    elseif file_exists(path_win) then
-        path = file_win
-    else
-        print("[WARNING] Failed to load plugin " .. base)
-        plugin_issues = true
-    end
-    plugins[#plugins + 1] = path
-end
-"#;
+pub const PLUGIN_BOOTSTRAP: &str = include_str!("plugin/bootstrap.lua");
 
 /// This contains the code for running the plugins
-pub const PLUGIN_RUN: &str = "
-global_event_mapping = {}
-
-function merge_event_mapping()
-    for key, f in pairs(event_mapping) do
-        if global_event_mapping[key] ~= nil then
-            table.insert(global_event_mapping[key], f)
-        else
-            global_event_mapping[key] = {f,}
-        end
-    end
-    event_mapping = {}
-end
-
-for c, path in ipairs(plugins) do
-    merge_event_mapping()
-    dofile(path)
-end
-merge_event_mapping()
-
-if plugin_issues then
-    print(\"Various plug-ins failed to load\")
-    print(\"You may download these plug-ins from the ox git repository (in the plugins folder)\")
-    print(\"https://github.com/curlpipe/ox\")
-    print(\"\")
-    print(\"Alternatively, you may silence these warnings\\nby removing the load_plugin() lines in your configuration file\\nfor the missing plug-ins that are listed above\")
-end
-";
+pub const PLUGIN_RUN: &str = include_str!("plugin/run.lua");
 
 /// This contains the code for handling a key binding
 pub fn run_key(key: &str) -> String {
     format!(
         "
+        globalevent = (global_event_mapping[\"*\"] or {{}})
+        for _, f in ipairs(globalevent) do
+            f()
+        end
         key = (global_event_mapping[\"{key}\"] or error(\"key not bound\"))
         for _, f in ipairs(key) do
             f()
         end
-    "
+        "
     )
 }
 
@@ -108,6 +57,7 @@ pub struct Config {
     pub status_line: Rc<RefCell<StatusLine>>,
     pub tab_line: Rc<RefCell<TabLine>>,
     pub greeting_message: Rc<RefCell<GreetingMessage>>,
+    pub help_message: Rc<RefCell<HelpMessage>>,
     pub terminal: Rc<RefCell<TerminalConfig>>,
     pub document: Rc<RefCell<DocumentConfig>>,
 }
@@ -119,6 +69,7 @@ impl Config {
         let syntax_highlighting = Rc::new(RefCell::new(SyntaxHighlighting::default()));
         let line_numbers = Rc::new(RefCell::new(LineNumbers::default()));
         let greeting_message = Rc::new(RefCell::new(GreetingMessage::default()));
+        let help_message = Rc::new(RefCell::new(HelpMessage::default()));
         let colors = Rc::new(RefCell::new(Colors::default()));
         let status_line = Rc::new(RefCell::new(StatusLine::default()));
         let tab_line = Rc::new(RefCell::new(TabLine::default()));
@@ -130,6 +81,7 @@ impl Config {
         lua.globals().set("line_numbers", line_numbers.clone())?;
         lua.globals()
             .set("greeting_message", greeting_message.clone())?;
+        lua.globals().set("help_message", help_message.clone())?;
         lua.globals().set("status_line", status_line.clone())?;
         lua.globals().set("tab_line", tab_line.clone())?;
         lua.globals().set("colors", colors.clone())?;
@@ -140,6 +92,7 @@ impl Config {
             syntax_highlighting,
             line_numbers,
             greeting_message,
+            help_message,
             tab_line,
             status_line,
             colors,
@@ -152,20 +105,21 @@ impl Config {
     pub fn read(&mut self, path: String, lua: &Lua) -> Result<()> {
         // Load the default config to start with
         lua.load(DEFAULT_CONFIG).exec()?;
+        // Reset plugin status based on built-in configuration file
+        lua.load("plugins = {}").exec()?;
+        lua.load("builtins = {}").exec()?;
 
         // Judge pre-user config state
         let status_parts = self.status_line.borrow().parts.len();
 
         // Attempt to read config file from home directory
+        let mut user_provided_config = false;
         if let Ok(path) = shellexpand::full(&path) {
             if let Ok(config) = std::fs::read_to_string(path.to_string()) {
                 // Update configuration with user-defined values
                 lua.load(config).exec()?;
-            } else {
-                return Err(OxError::Config("Not Found".to_string()));
+                user_provided_config = true;
             }
-        } else {
-            return Err(OxError::Config("Not Found".to_string()));
         }
 
         // Remove any default values if necessary
@@ -173,7 +127,48 @@ impl Config {
             self.status_line.borrow_mut().parts.drain(0..status_parts);
         }
 
-        Ok(())
+        // Determine whether or not to load built-in plugins
+        let mut builtins: HashMap<&str, &str> = HashMap::default();
+        builtins.insert("pairs.lua", PAIRS);
+        builtins.insert("autoindent.lua", AUTOINDENT);
+        for (name, code) in builtins.iter() {
+            if self.load_bi(name, user_provided_config, &lua) {
+                lua.load(*code).exec()?;
+            }
+        }
+
+        if user_provided_config {
+            Ok(())
+        } else {
+            Err(OxError::Config("Not Found".to_string()))
+        }
+    }
+
+    /// Decide whether to load a built-in plugin
+    pub fn load_bi(&self, name: &str, user_provided_config: bool, lua: &Lua) -> bool {
+        if !user_provided_config {
+            // Load when the user hasn't provided a configuration file
+            true
+        } else {
+            // Get list of user-loaded plug-ins
+            let plugins: Vec<String> = lua
+                .globals()
+                .get::<_, LuaTable>("builtins")
+                .unwrap()
+                .sequence_values()
+                .filter_map(std::result::Result::ok)
+                .collect();
+            // If the user wants to load the plug-in but it isn't available
+            if let Some(idx) = plugins.iter().position(|p| p.ends_with(name)) {
+                // User wants the plug-in
+                let path = &plugins[idx];
+                // true if plug-in isn't avilable
+                !std::path::Path::new(path).exists()
+            } else {
+                // User doesn't want the plug-in
+                false
+            }
+        }
     }
 }
 
@@ -389,13 +384,32 @@ impl Default for GreetingMessage {
 
 impl GreetingMessage {
     /// Take the configuration information and render the greeting message
-    pub fn render(&self, colors: &Colors) -> Result<String> {
+    pub fn render(&self, lua: &Lua, colors: &Colors) -> Result<String> {
         let highlight = Fg(colors.highlight.to_color()?).to_string();
         let editor_fg = Fg(colors.editor_fg.to_color()?).to_string();
         let mut result = self.format.clone();
         result = result.replace("{version}", &VERSION).to_string();
         result = result.replace("{highlight_start}", &highlight).to_string();
         result = result.replace("{highlight_end}", &editor_fg).to_string();
+        // Find functions to call and substitute in
+        let mut searcher = Searcher::new(r"\{[A-Za-z_][A-Za-z0-9_]*\}");
+        while let Some(m) = searcher.lfind(&result) {
+            let name = m
+                .text
+                .chars()
+                .skip(1)
+                .take(m.text.chars().count().saturating_sub(2))
+                .collect::<String>();
+            if let Ok(func) = lua.globals().get::<String, LuaFunction>(name) {
+                if let Ok(r) = func.call::<(), LuaString>(()) {
+                    result = result.replace(&m.text, r.to_str().unwrap_or(""));
+                } else {
+                    break;
+                }
+            } else {
+                break;
+            }
+        }
         Ok(result)
     }
 }
@@ -415,6 +429,69 @@ impl LuaUserData for GreetingMessage {
     }
 }
 
+/// For storing configuration information related to the help message
+#[derive(Debug)]
+pub struct HelpMessage {
+    pub enabled: bool,
+    pub format: String,
+}
+
+impl Default for HelpMessage {
+    fn default() -> Self {
+        Self {
+            enabled: true,
+            format: "".to_string(),
+        }
+    }
+}
+
+impl HelpMessage {
+    /// Take the configuration information and render the help message
+    pub fn render(&self, lua: &Lua, colors: &Colors) -> Result<Vec<String>> {
+        let highlight = Fg(colors.highlight.to_color()?).to_string();
+        let editor_fg = Fg(colors.editor_fg.to_color()?).to_string();
+        let mut result = self.format.clone();
+        result = result.replace("{version}", &VERSION).to_string();
+        result = result.replace("{highlight_start}", &highlight).to_string();
+        result = result.replace("{highlight_end}", &editor_fg).to_string();
+        // Find functions to call and substitute in
+        let mut searcher = Searcher::new(r"\{[A-Za-z_][A-Za-z0-9_]*\}");
+        while let Some(m) = searcher.lfind(&result) {
+            let name = m
+                .text
+                .chars()
+                .skip(1)
+                .take(m.text.chars().count().saturating_sub(2))
+                .collect::<String>();
+            if let Ok(func) = lua.globals().get::<String, LuaFunction>(name) {
+                if let Ok(r) = func.call::<(), LuaString>(()) {
+                    result = result.replace(&m.text, r.to_str().unwrap_or(""));
+                } else {
+                    break;
+                }
+            } else {
+                break;
+            }
+        }
+        Ok(result.split('\n').map(|l| l.to_string()).collect())
+    }
+}
+
+impl LuaUserData for HelpMessage {
+    fn add_fields<'lua, F: LuaUserDataFields<'lua, Self>>(fields: &mut F) {
+        fields.add_field_method_get("enabled", |_, this| Ok(this.enabled));
+        fields.add_field_method_set("enabled", |_, this, value| {
+            this.enabled = value;
+            Ok(())
+        });
+        fields.add_field_method_get("format", |_, this| Ok(this.format.clone()));
+        fields.add_field_method_set("format", |_, this, value| {
+            this.format = value;
+            Ok(())
+        });
+    }
+}
+
 /// For storing configuration information related to the status line
 #[derive(Debug)]
 pub struct TabLine {
@@ -499,7 +576,13 @@ impl StatusLine {
         let file_extension = get_file_ext(&path).unwrap_or_else(|| "".to_string());
         let absolute_path = get_absolute_path(&path).unwrap_or_else(|| "[No Name]".to_string());
         let file_name = get_file_name(&path).unwrap_or_else(|| "[No Name]".to_string());
-        let file_type = filetype(&file_extension).unwrap_or_else(|| file_extension.to_string());
+        let file_type = filetype(&file_extension).unwrap_or_else(|| {
+            if file_extension.is_empty() {
+                "Unknown".to_string()
+            } else {
+                file_extension.to_string()
+            }
+        });
         let icon = icon(&filetype(&file_extension).unwrap_or_else(|| "".to_string()));
         let modified = if editor.doc().modified { "[+]" } else { "" };
         let cursor_y = (editor.doc().loc().y + 1).to_string();
@@ -1070,6 +1153,7 @@ impl LuaUserData for DocumentConfig {
 
 impl LuaUserData for Editor {
     fn add_fields<'lua, F: LuaUserDataFields<'lua, Self>>(fields: &mut F) {
+        fields.add_field_method_get("pasting", |_, editor| Ok(editor.paste_flag));
         fields.add_field_method_get("cursor", |_, editor| {
             let loc = editor.doc().char_loc();
             Ok(LuaLoc {
@@ -1088,7 +1172,8 @@ impl LuaUserData for Editor {
         fields.add_field_method_get("version", |_, _| Ok(VERSION));
         fields.add_field_method_get("current_document_id", |_, editor| Ok(editor.ptr));
         fields.add_field_method_get("document_count", |_, editor| Ok(editor.doc.len()));
-        fields.add_field_method_get("help_visible", |_, editor| Ok(editor.help));
+        // DEPRECIATED
+        fields.add_field_method_get("help_visible", |_, _| Ok(false));
         fields.add_field_method_get("document_type", |_, editor| {
             let ext = editor
                 .doc()
@@ -1214,6 +1299,14 @@ impl LuaUserData for Editor {
             update_highlighter(editor);
             Ok(())
         });
+        methods.add_method_mut("cut", |_, editor, ()| {
+            if let Err(err) = editor.cut() {
+                editor.feedback = Feedback::Error(err.to_string());
+            } else {
+                editor.feedback = Feedback::Info("Text cut to clipboard".to_owned());
+            }
+            Ok(())
+        });
         methods.add_method_mut("copy", |_, editor, ()| {
             if let Err(err) = editor.copy() {
                 editor.feedback = Feedback::Error(err.to_string());
@@ -1222,12 +1315,6 @@ impl LuaUserData for Editor {
             }
             Ok(())
         });
-        methods.add_method_mut("paste", |_, editor, ()| {
-            if let Err(err) = editor.paste() {
-                editor.feedback = Feedback::Error(err.to_string());
-            }
-            Ok(())
-        });
         methods.add_method_mut("move_home", |_, editor, ()| {
             editor.doc_mut().move_home();
             update_highlighter(editor);
@@ -1461,14 +1548,10 @@ impl LuaUserData for Editor {
             update_highlighter(editor);
             Ok(())
         });
-        methods.add_method_mut("hide_help_message", |_, editor, ()| {
-            editor.help = false;
-            Ok(())
-        });
-        methods.add_method_mut("show_help_message", |_, editor, ()| {
-            editor.help = true;
-            Ok(())
-        });
+        // DEPRECIATED
+        methods.add_method_mut("hide_help_message", |_, _, ()| Ok(()));
+        // DEPRECIATED
+        methods.add_method_mut("show_help_message", |_, _, ()| Ok(()));
         methods.add_method_mut("set_read_only", |_, editor, status: bool| {
             editor.doc_mut().read_only = status;
             Ok(())
