@@ -1,6 +1,6 @@
 use crate::config::{key_to_string, Config};
 use crate::error::{OxError, Result};
-use crate::ui::{size, Feedback, Terminal, HELP_TEXT};
+use crate::ui::{size, Feedback, Terminal};
 use crossterm::{
     event::{read, Event as CEvent, KeyCode as KCode, KeyModifiers as KMod},
     style::{Attribute, SetAttribute, SetBackgroundColor as Bg, SetForegroundColor as Fg},
@@ -34,8 +34,6 @@ pub struct Editor {
     pub active: bool,
     /// true if the editor should show a greeting message on next render
     pub greet: bool,
-    /// Whether or not to show the help message
-    pub help: bool,
     /// The feedback message to display below the status line
     pub feedback: Feedback,
     /// Will be some if there is an outstanding command to be run
@@ -46,6 +44,8 @@ pub struct Editor {
     push_down: usize,
     /// Used to cache the location of the configuration file
     pub config_path: String,
+    /// This is a handy place to figure out if the user is currently pasting something or not
+    pub paste_flag: bool,
 }
 
 impl Editor {
@@ -59,7 +59,6 @@ impl Editor {
             config,
             active: true,
             greet: false,
-            help: false,
             needs_rerender: true,
             highlighter: vec![],
             feedback: Feedback::None,
@@ -67,6 +66,7 @@ impl Editor {
             last_active: Instant::now(),
             push_down: 1,
             config_path: "~/.oxrc".to_string(),
+            paste_flag: false,
         })
     }
 
@@ -136,6 +136,7 @@ impl Editor {
             .get_highlighter(&ext);
         highlighter.run(&doc.lines);
         self.highlighter.push(highlighter);
+        doc.undo_mgmt.saved();
         // Add document to documents
         self.doc.push(doc);
         Ok(())
@@ -247,15 +248,19 @@ impl Editor {
         };
         match event {
             CEvent::Key(key) => {
-                // Check period of inactivity and commit events (for undo/redo) if over 10secs
+                // Check period of inactivity
                 let end = Instant::now();
-                let inactivity = end.duration_since(self.last_active).as_secs() as usize;
-                if inactivity > self.config.document.borrow().undo_period {
-                    self.doc_mut().event_mgmt.commit();
+                let inactivity = end.duration_since(self.last_active).as_millis() as usize;
+                if inactivity > self.config.document.borrow().undo_period * 1000 {
+                    self.doc_mut().commit();
                 }
+                // Predict whether the user is currently pasting text (based on rapid activity)
+                self.paste_flag = inactivity < 5;
+                // Register this activity
                 self.last_active = Instant::now();
                 // Editing - these key bindings can't be modified (only added to)!
                 match (key.modifiers, key.code) {
+                    // Core key bindings (non-configurable behaviour)
                     (KMod::SHIFT | KMod::NONE, KCode::Char(ch)) => self.character(ch)?,
                     (KMod::NONE, KCode::Tab) => self.character('\t')?,
                     (KMod::NONE, KCode::Backspace) => self.backspace()?,
@@ -327,9 +332,9 @@ impl Editor {
         self.render_status_line(&lua, w, h)?;
         // Render greeting or help message if applicable
         if self.greet {
-            self.render_greeting(w, h)?;
-        } else if self.help {
-            self.render_help_message(w, h)?;
+            self.render_greeting(lua, w, h)?;
+        } else if self.config.help_message.borrow().enabled {
+            self.render_help_message(lua, w, h)?;
         }
         // Render feedback line
         self.render_feedback_line(w, h)?;
@@ -402,17 +407,14 @@ impl Editor {
                         TokOpt::None(text) => text,
                     };
                     for c in text.chars() {
-                        let is_selected = self.doc().is_loc_selected(Loc { y: idx, x: x_pos });
+                        let at_x = self.doc().character_idx(&Loc { y: idx, x: x_pos });
+                        let is_selected = self.doc().is_loc_selected(Loc { y: idx, x: at_x });
                         if is_selected {
                             write!(
                                 self.terminal.stdout,
-                                "{}",
-                                Bg(self.config.colors.borrow().selection_bg.to_color()?)
-                            )?;
-                            write!(
-                                self.terminal.stdout,
-                                "{}",
-                                Fg(self.config.colors.borrow().selection_fg.to_color()?)
+                                "{}{}",
+                                Bg(self.config.colors.borrow().selection_bg.to_color()?),
+                                Fg(self.config.colors.borrow().selection_fg.to_color()?),
                             )?;
                         } else {
                             write!(
@@ -496,21 +498,20 @@ impl Editor {
     }
 
     /// Render the greeting message
-    fn render_help_message(&mut self, w: usize, h: usize) -> Result<()> {
-        let color = self.config.colors.borrow().highlight.to_color()?;
-        let editor_fg = self.config.colors.borrow().editor_fg.to_color()?;
-        let message: Vec<&str> = HELP_TEXT.split('\n').collect();
+    fn render_help_message(&mut self, lua: &Lua, w: usize, h: usize) -> Result<()> {
+        let colors = self.config.colors.borrow();
+        let message = self.config.help_message.borrow().render(lua, &colors)?;
         for (c, line) in message.iter().enumerate().take(h.saturating_sub(h / 4)) {
             self.terminal.goto(w.saturating_sub(30), h / 4 + c + 1)?;
-            write!(self.terminal.stdout, "{}{line}{}", Fg(color), Fg(editor_fg))?;
+            write!(self.terminal.stdout, "{line}")?;
         }
         Ok(())
     }
 
     /// Render the help message
-    fn render_greeting(&mut self, w: usize, h: usize) -> Result<()> {
+    fn render_greeting(&mut self, lua: &Lua, w: usize, h: usize) -> Result<()> {
         let colors = self.config.colors.borrow();
-        let greeting = self.config.greeting_message.borrow().render(&colors)?;
+        let greeting = self.config.greeting_message.borrow().render(lua, &colors)?;
         let message: Vec<&str> = greeting.split('\n').collect();
         for (c, line) in message.iter().enumerate().take(h.saturating_sub(h / 4)) {
             self.terminal.goto(4, h / 4 + c + 1)?;
@@ -598,15 +599,11 @@ impl Editor {
         self.terminal.copy(&selected_text)
     }
 
-    /// Paste the selected text
-    pub fn paste(&mut self) -> Result<()> {
-        let clip = self.terminal.paste();
-        for ch in clip.unwrap_or_else(|| "".to_string()).chars() {
-            if let Err(err) = self.character(ch) {
-                self.feedback = Feedback::Error(err.to_string());
-            }
-        }
-        self.update_highlighter()?;
+    /// Cut the selected text
+    pub fn cut(&mut self) -> Result<()> {
+        self.copy()?;
+        self.doc_mut().remove_selection();
+        self.reload_highlight();
         Ok(())
     }
 
@@ -703,6 +700,10 @@ impl Editor {
     /// Insert a character into the document, creating a new row if editing
     /// on the last line of the document
     pub fn character(&mut self, ch: char) -> Result<()> {
+        if !self.doc().is_selection_empty() {
+            self.doc_mut().remove_selection();
+            self.reload_highlight();
+        }
         self.new_row()?;
         // Handle the character insertion
         if ch == '\n' {
@@ -714,7 +715,7 @@ impl Editor {
         }
         // Commit to event stack (for undo/redo if the character is a space)
         if ch == ' ' {
-            self.doc_mut().event_mgmt.commit();
+            self.doc_mut().commit();
         }
         Ok(())
     }
@@ -722,7 +723,7 @@ impl Editor {
     /// Handle the return key
     pub fn enter(&mut self) -> Result<()> {
         // When the return key is pressed, we want to commit to the undo/redo stack
-        self.doc_mut().event_mgmt.commit();
+        self.doc_mut().commit();
         // Perform the changes
         if self.doc().loc().y != self.doc().len_lines() {
             // Enter pressed in the start, middle or end of the line
@@ -741,6 +742,14 @@ impl Editor {
 
     /// Handle the backspace key
     pub fn backspace(&mut self) -> Result<()> {
+        if !self.doc().is_selection_empty() {
+            self.doc_mut().commit();
+            self.doc_mut().undo_mgmt.set_dirty();
+            self.doc_mut().remove_selection();
+            // Removing a selection is significant and worth an undo commit
+            self.reload_highlight();
+            return Ok(());
+        }
         let mut c = self.doc().char_ptr;
         let on_first_line = self.doc().loc().y == 0;
         let out_of_range = self.doc().out_of_range(0, self.doc().loc().y).is_err();
@@ -799,7 +808,7 @@ impl Editor {
     /// Delete the current line
     pub fn delete_line(&mut self) -> Result<()> {
         // Commit events to event manager (for undo / redo)
-        self.doc_mut().event_mgmt.commit();
+        self.doc_mut().commit();
         // Delete the line
         if self.doc().loc().y < self.doc().len_lines() {
             let y = self.doc().loc().y;
@@ -939,7 +948,7 @@ impl Editor {
     /// Replace an instance in a document
     fn do_replace(&mut self, into: &str, text: &str) -> Result<()> {
         // Commit events to event manager (for undo / redo)
-        self.doc_mut().event_mgmt.commit();
+        self.doc_mut().commit();
         // Do the replacement
         let loc = self.doc().char_loc();
         self.doc_mut().replace(loc, text, into)?;
@@ -953,7 +962,7 @@ impl Editor {
     /// Replace all instances in a document
     fn do_replace_all(&mut self, target: &str, into: &str) {
         // Commit events to event manager (for undo / redo)
-        self.doc_mut().event_mgmt.commit();
+        self.doc_mut().commit();
         // Replace everything top to bottom
         self.doc_mut().move_to(&Loc::at(0, 0));
         while let Some(mtch) = self.doc_mut().next_match(target, 1) {
@@ -963,45 +972,31 @@ impl Editor {
         }
     }
 
+    /// Reload the whole document in the highlighter
+    fn reload_highlight(&mut self) {
+        self.highlighter[self.ptr].run(&self.doc[self.ptr].lines);
+    }
+
     /// Perform redo action
     pub fn redo(&mut self) -> Result<()> {
         let result = Ok(self.doc_mut().redo()?);
-        let mut affected_lines = vec![];
-        if let Some(patch) = self.doc().event_mgmt.undo.last() {
-            for event in patch {
-                affected_lines.push(event.clone().loc().y);
-            }
-        }
-        affected_lines.sort();
-        affected_lines.dedup();
-        for line in affected_lines {
-            self.highlighter[self.ptr].edit(line, &self.doc[self.ptr].lines[line]);
-        }
+        self.reload_highlight();
         result
     }
 
     /// Perform undo action
     pub fn undo(&mut self) -> Result<()> {
         let result = Ok(self.doc_mut().undo()?);
-        let mut affected_lines = vec![];
-        if let Some(patch) = self.doc().event_mgmt.redo.last() {
-            for event in patch {
-                affected_lines.push(event.clone().loc().y);
-            }
-        }
-        affected_lines.sort();
-        affected_lines.dedup();
-        for line in affected_lines {
-            self.highlighter[self.ptr].edit(line, &self.doc[self.ptr].lines[line]);
-        }
+        self.reload_highlight();
         result
     }
 
     /// save the document to the disk
     pub fn save(&mut self) -> Result<()> {
-        self.doc_mut().save()?;
         // Commit events to event manager (for undo / redo)
-        self.doc_mut().event_mgmt.commit();
+        self.doc_mut().commit();
+        // Perform the save
+        self.doc_mut().save()?;
         // All done
         self.feedback = Feedback::Info("Document saved successfully".to_string());
         Ok(())
@@ -1022,7 +1017,7 @@ impl Editor {
             self.doc_mut().modified = false;
         }
         // Commit events to event manager (for undo / redo)
-        self.doc_mut().event_mgmt.commit();
+        self.doc_mut().commit();
         // All done
         self.feedback = Feedback::Info(format!("Document saved as {file_name} successfully"));
         Ok(())
@@ -1033,7 +1028,7 @@ impl Editor {
         for doc in self.doc.iter_mut() {
             doc.save()?;
             // Commit events to event manager (for undo / redo)
-            doc.event_mgmt.commit();
+            doc.commit();
         }
         self.feedback = Feedback::Info(format!("Saved all documents"));
         Ok(())
