@@ -1,6 +1,6 @@
 use crate::config::{Colors, TerminalConfig};
-use crate::error::{OxError, Result};
-use copypasta_ext::ClipboardProviderExt;
+use crate::error::Result;
+use base64::prelude::*;
 use crossterm::{
     cursor::{Hide, MoveTo, Show},
     event::{
@@ -19,31 +19,6 @@ use std::cell::RefCell;
 use std::io::{stdout, Stdout, Write};
 use std::rc::Rc;
 
-/// Constant that shows the help message
-pub const HELP_TEXT: &str = "
-   Default Key Bindings:       
-   Ctrl + N:   New             
-   Ctrl + O:   Open            
-   Ctrl + Q:   Quit            
-   Ctrl + S:   Save            
-   Alt  + W:   Save as         
-   Alt  + A:   Save all        
-   Ctrl + A:   Select All      
-   Ctrl + C:   Copy            
-   Ctrl + V:   Paste           
-   Ctrl + Z:   Undo            
-   Ctrl + Y:   Redo            
-   Ctrl + F:   Find            
-   Ctrl + R:   Replace         
-   Ctrl + W:   Delete Word     
-   Ctrl + D:   Delete Line     
-   Ctrl + K:   Command Line    
-   Alt + Up:   Move line up    
-   Alt + Down: Move line down  
-   Alt + ->:   Next Tab        
-   Alt + <-:   Previous Tab    
-";
-
 /// Gets the size of the terminal
 pub fn size() -> Result<Size> {
     let (w, h) = terminal::size()?;
@@ -106,7 +81,6 @@ impl Feedback {
 pub struct Terminal {
     pub stdout: Stdout,
     pub config: Rc<RefCell<TerminalConfig>>,
-    pub clipboard: Option<Box<dyn ClipboardProviderExt>>,
 }
 
 impl Terminal {
@@ -114,7 +88,6 @@ impl Terminal {
         Terminal {
             stdout: stdout(),
             config,
-            clipboard: copypasta_ext::try_context(),
         }
     }
 
@@ -187,20 +160,11 @@ impl Terminal {
 
     /// Put text into the clipboard
     pub fn copy(&mut self, text: &str) -> Result<()> {
-        let result = self
-            .clipboard
-            .as_deref_mut()
-            .ok_or(OxError::Clipboard)?
-            .set_contents(text.to_string());
-        if result.is_err() {
-            Err(OxError::Clipboard)
-        } else {
-            Ok(())
-        }
-    }
-
-    /// Get text from the clipboard
-    pub fn paste(&mut self) -> Option<String> {
-        self.clipboard.as_deref_mut()?.get_contents().ok()
+        write!(
+            self.stdout,
+            "\x1b]52;c;{}\x1b\\",
+            BASE64_STANDARD.encode(text)
+        )?;
+        Ok(())
     }
 }
